@@ -1,14 +1,15 @@
 """Base library for database interaction with PgStac."""
+import atexit
+import logging
 import time
 from types import TracebackType
-from typing import Any, List, Optional, Tuple, Type, Union, Generator
+from typing import Any, Generator, List, Optional, Tuple, Type, Union
+
 import orjson
 import psycopg
 from psycopg import Connection, sql
 from psycopg.types.json import set_json_dumps, set_json_loads
 from psycopg_pool import ConnectionPool
-import atexit
-import logging
 from pydantic import BaseSettings
 from tenacity import retry, retry_if_exception_type, stop_after_attempt
 
@@ -84,10 +85,6 @@ class PgstacDB:
                 max_waiting=settings.db_max_queries,
                 max_idle=settings.db_max_idle,
                 num_workers=settings.db_num_workers,
-                kwargs={
-                    "options": "-c search_path=pgstac,public"
-                    " -c application_name=pypgstac"
-                },
             )
         return self.pool
 
@@ -109,6 +106,24 @@ class PgstacDB:
             if self.debug:
                 self.connection.add_notice_handler(pg_notice_handler)
             atexit.register(self.disconnect)
+            self.connection.execute(
+                """
+                    SELECT
+                        CASE
+                        WHEN
+                        current_setting('search_path', false) ~* '\\mpgstac\\M'
+                        THEN current_setting('search_path', false)
+                        ELSE set_config(
+                            'search_path',
+                            'pgstac,' || current_setting('search_path', false),
+                            false
+                            )
+                        END
+                    ;
+                    SET application_name TO 'pgstac';
+                """,
+                prepare=False,
+            )
         return self.connection
 
     def wait(self) -> None:
@@ -127,17 +142,17 @@ class PgstacDB:
     def disconnect(self) -> None:
         """Disconnect from database."""
         try:
-            if self.commit_on_exit:
-                if self.connection is not None:
+            if self.connection is not None:
+                if self.commit_on_exit:
                     self.connection.commit()
-                if self.connection is not None:
+                else:
                     self.connection.rollback()
-        except:
+        except Exception:
             pass
         try:
             if self.pool is not None and self.connection is not None:
                 self.pool.putconn(self.connection)
-        except:
+        except Exception:
             pass
 
         self.connection = None
@@ -215,7 +230,7 @@ class PgstacDB:
                 """
                 SELECT version from pgstac.migrations
                 order by datetime desc, version desc limit 1;
-                """
+                """,
             )
             logger.debug(f"VERSION: {version}")
             if isinstance(version, bytes):
@@ -234,7 +249,7 @@ class PgstacDB:
         version = self.query_one(
             """
             SHOW server_version;
-            """
+            """,
         )
         logger.debug(f"PG VERSION: {version}.")
         if isinstance(version, bytes):
