@@ -1,31 +1,32 @@
 """Utilities to bulk load data into pgstac from json/ndjson."""
 import contextlib
-from datetime import datetime
 import itertools
 import logging
-from pathlib import Path
 import sys
 import time
 from dataclasses import dataclass
-from functools import lru_cache
+from datetime import datetime
+from enum import Enum
+from pathlib import Path
 from typing import (
     Any,
     BinaryIO,
     Dict,
+    Generator,
     Iterable,
     Iterator,
     Optional,
+    TextIO,
     Tuple,
     Union,
-    Generator,
-    TextIO,
 )
+
 import orjson
 import psycopg
+from cachetools.func import lru_cache
 from orjson import JSONDecodeError
 from plpygis.geometry import Geometry
 from psycopg import sql
-from psycopg.types.range import Range
 from smart_open import open
 from tenacity import (
     retry,
@@ -33,12 +34,11 @@ from tenacity import (
     stop_after_attempt,
     wait_random_exponential,
 )
-
+from version_parser import Version as V
 
 from .db import PgstacDB
 from .hydration import dehydrate
 from .version import __version__
-from enum import Enum
 
 logger = logging.getLogger(__name__)
 
@@ -83,7 +83,7 @@ class Methods(str, Enum):
 
 @contextlib.contextmanager
 def open_std(
-    filename: str, mode: str = "r", *args: Any, **kwargs: Any
+    filename: str, mode: str = "r", *args: Any, **kwargs: Any,
 ) -> Generator[Any, None, None]:
     """Open files and i/o streams transparently."""
     fh: Union[TextIO, BinaryIO]
@@ -93,14 +93,8 @@ def open_std(
         or filename == "stdin"
         or filename == "stdout"
     ):
-        if "r" in mode:
-            stream = sys.stdin
-        else:
-            stream = sys.stdout
-        if "b" in mode:
-            fh = stream.buffer
-        else:
-            fh = stream
+        stream = sys.stdin if "r" in mode else sys.stdout
+        fh = stream.buffer if "b" in mode else stream
         close = False
     else:
         fh = open(filename, mode, *args, **kwargs)
@@ -161,10 +155,21 @@ class Loader:
         self._partition_cache: Dict[str, Partition] = {}
 
     def check_version(self) -> None:
-        if self.db.version != __version__:
+        db_version = self.db.version
+        if db_version is None:
+            raise Exception("Failed to detect the target database version.")
+
+        v1 = V(db_version)
+        v2 = V(__version__)
+        if (v1.get_major_version(), v1.get_minor_version()) != (
+            v2.get_major_version(),
+            v2.get_minor_version(),
+        ):
             raise Exception(
-                f"pypgstac version {__version__} is not compatible with the target"
+                f"pypgstac version {__version__}"
+                " is not compatible with the target"
                 f" database version {self.db.version}."
+                f" database version {db_version}.",
             )
 
     @lru_cache(maxsize=128)
@@ -180,7 +185,7 @@ class Loader:
             raise Exception(f"Error getting info for {collection_id}.")
         if key is None:
             raise Exception(
-                f"Collection {collection_id} is not present in the database"
+                f"Collection {collection_id} is not present in the database",
             )
         logger.debug(f"Found {collection_id} with base_item {base_item}")
         return base_item, key, partition_trunc
@@ -203,7 +208,7 @@ class Loader:
                     DROP TABLE IF EXISTS tmp_collections;
                     CREATE TEMP TABLE tmp_collections
                     (content jsonb) ON COMMIT DROP;
-                    """
+                    """,
                 )
                 with cur.copy("COPY tmp_collections (content) FROM stdin;") as copy:
                     for collection in read_json(file):
@@ -216,7 +221,7 @@ class Loader:
                         """
                         INSERT INTO collections (content)
                         SELECT content FROM tmp_collections;
-                        """
+                        """,
                     )
                     logger.debug(cur.statusmessage)
                     logger.debug(f"Rows affected: {cur.rowcount}")
@@ -229,7 +234,7 @@ class Loader:
                         INSERT INTO collections (content)
                         SELECT content FROM tmp_collections
                         ON CONFLICT DO NOTHING;
-                        """
+                        """,
                     )
                     logger.debug(cur.statusmessage)
                     logger.debug(f"Rows affected: {cur.rowcount}")
@@ -240,14 +245,14 @@ class Loader:
                         SELECT content FROM tmp_collections
                         ON CONFLICT (id) DO
                         UPDATE SET content=EXCLUDED.content;
-                        """
+                        """,
                     )
                     logger.debug(cur.statusmessage)
                     logger.debug(f"Rows affected: {cur.rowcount}")
                 else:
                     raise Exception(
                         "Available modes are insert, ignore, and upsert."
-                        f"You entered {insert_mode}."
+                        f"You entered {insert_mode}.",
                     )
 
     @retry(
@@ -273,20 +278,13 @@ class Loader:
         with conn.cursor() as cur:
             if partition.requires_update:
                 with conn.transaction():
-                    cur.execute(
-                        "SELECT * FROM partitions WHERE name = %s FOR UPDATE;",
-                        (partition.name,),
-                    )
                     cur.execute(
                         """
-                        INSERT INTO partitions
-                        (collection, datetime_range, end_datetime_range)
-                        VALUES
-                            (%s, tstzrange(%s, %s, '[]'), tstzrange(%s,%s, '[]'))
-                        ON CONFLICT (name) DO UPDATE SET
-                            datetime_range = EXCLUDED.datetime_range,
-                            end_datetime_range = EXCLUDED.end_datetime_range
-                        ;
+                        SELECT check_partition(
+                            %s,
+                            tstzrange(%s, %s, '[]'),
+                            tstzrange(%s, %s, '[]')
+                        );
                     """,
                         (
                             partition.collection,
@@ -296,15 +294,17 @@ class Loader:
                             partition.end_datetime_range_max,
                         ),
                     )
+
                     logger.debug(
                         f"Adding or updating partition {partition.name} "
-                        f"took {time.perf_counter() - t}s"
+                        f"took {time.perf_counter() - t}s",
                     )
                 partition.requires_update = False
             else:
                 logger.debug(f"Partition {partition.name} does not require an update.")
 
             with conn.transaction():
+
                 t = time.perf_counter()
                 if insert_mode in (
                     None,
@@ -316,8 +316,8 @@ class Loader:
                             COPY {}
                             (id, collection, datetime, end_datetime, geometry, content)
                             FROM stdin;
-                            """
-                        ).format(sql.Identifier(partition.name))
+                            """,
+                        ).format(sql.Identifier(partition.name)),
                     ) as copy:
                         for item in items:
                             item.pop("partition")
@@ -329,7 +329,7 @@ class Loader:
                                     item["end_datetime"],
                                     item["geometry"],
                                     item["content"],
-                                )
+                                ),
                             )
                     logger.debug(cur.statusmessage)
                     logger.debug(f"Rows affected: {cur.rowcount}")
@@ -344,14 +344,14 @@ class Loader:
                         DROP TABLE IF EXISTS items_ingest_temp;
                         CREATE TEMP TABLE items_ingest_temp
                         ON COMMIT DROP AS SELECT * FROM items LIMIT 0;
-                        """
+                        """,
                     )
                     with cur.copy(
                         """
                         COPY items_ingest_temp
                         (id, collection, datetime, end_datetime, geometry, content)
                         FROM stdin;
-                        """
+                        """,
                     ) as copy:
                         for item in items:
                             item.pop("partition")
@@ -363,7 +363,7 @@ class Loader:
                                     item["end_datetime"],
                                     item["geometry"],
                                     item["content"],
-                                )
+                                ),
                             )
                     logger.debug(cur.statusmessage)
                     logger.debug(f"Copied rows: {cur.rowcount}")
@@ -372,8 +372,8 @@ class Loader:
                         sql.SQL(
                             """
                                 LOCK TABLE ONLY {} IN EXCLUSIVE MODE;
-                            """
-                        ).format(sql.Identifier(partition.name))
+                            """,
+                        ).format(sql.Identifier(partition.name)),
                     )
                     if insert_mode in (
                         Methods.ignore,
@@ -385,8 +385,8 @@ class Loader:
                                 INSERT INTO {}
                                 SELECT *
                                 FROM items_ingest_temp ON CONFLICT DO NOTHING;
-                                """
-                            ).format(sql.Identifier(partition.name))
+                                """,
+                            ).format(sql.Identifier(partition.name)),
                         )
                         logger.debug(cur.statusmessage)
                         logger.debug(f"Rows affected: {cur.rowcount}")
@@ -404,8 +404,8 @@ class Loader:
                                     content = EXCLUDED.content
                                 WHERE t IS DISTINCT FROM EXCLUDED
                                 ;
-                            """
-                            ).format(sql.Identifier(partition.name))
+                            """,
+                            ).format(sql.Identifier(partition.name)),
                         )
                         logger.debug(cur.statusmessage)
                         logger.debug(f"Rows affected: {cur.rowcount}")
@@ -418,27 +418,31 @@ class Loader:
                                         WHERE
                                             i.id = s.id
                                             AND i.collection = s.collection
-                                            AND i IS DISTINCT FROM s
-                                    RETURNING i.id, i.collection
                                 )
-                                INSERT INTO {}
-                                SELECT s.* FROM
-                                    items_ingest_temp s
-                                    JOIN deletes d
-                                    USING (id, collection);
+                                INSERT INTO {} AS t SELECT * FROM items_ingest_temp
+                                ON CONFLICT (id) DO UPDATE
+                                SET
+                                    datetime = EXCLUDED.datetime,
+                                    end_datetime = EXCLUDED.end_datetime,
+                                    geometry = EXCLUDED.geometry,
+                                    collection = EXCLUDED.collection,
+                                    content = EXCLUDED.content
+                                WHERE t IS DISTINCT FROM EXCLUDED
                                 ;
-                            """
-                            ).format(sql.Identifier(partition.name))
+                                """,
+                            ).format(sql.Identifier(partition.name)),
+
                         )
                         logger.debug(cur.statusmessage)
                         logger.debug(f"Rows affected: {cur.rowcount}")
                 else:
                     raise Exception(
                         "Available modes are insert, ignore, upsert, and delsert."
-                        f"You entered {insert_mode}."
+                        f"You entered {insert_mode}.",
                     )
+                cur.execute("SELECT update_partition_stats_q(%s);",(partition.name,))
         logger.debug(
-            f"Copying data for {partition} took {time.perf_counter() - t} seconds"
+            f"Copying data for {partition} took {time.perf_counter() - t} seconds",
         )
 
     def _partition_update(self, item: Dict[str, Any]) -> str:
@@ -468,32 +472,41 @@ class Loader:
             # Read the partition information from the database if it exists
             db_rows = list(
                 self.db.query(
-                    "SELECT datetime_range, end_datetime_range "
-                    "FROM partitions WHERE name=%s;",
+                    """
+                    SELECT
+                        nullif(lower(constraint_dtrange),'-infinity')
+                            as datetime_range_min,
+                        nullif(upper(constraint_dtrange),'infinity')
+                            as datetime_range_max,
+                        nullif(lower(constraint_edtrange),'-infinity')
+                            as end_datetime_range_min,
+                        nullif(upper(constraint_edtrange),'infinity')
+                            as end_datetime_range_max
+                    FROM partitions WHERE partition=%s;
+                    """,
                     [partition_name],
-                )
+                ),
             )
             if db_rows:
-                datetime_range: Optional[Range[datetime]] = db_rows[0][0]
-                end_datetime_range: Optional[Range[datetime]] = db_rows[0][1]
+                datetime_range_min: Optional[datetime] = db_rows[0][0] or datetime.min
+                datetime_range_max: Optional[datetime] = db_rows[0][1] or datetime.max
+                end_datetime_range_min: Optional[datetime] = (
+                    db_rows[0][2] or datetime.min
+                )
+                end_datetime_range_max: Optional[datetime] = (
+                    db_rows[0][3] or datetime.max
+                )
+
+                partition = Partition(
+                    name=partition_name,
+                    collection=item["collection"],
+                    datetime_range_min=datetime_range_min.isoformat(),
+                    datetime_range_max=datetime_range_max.isoformat(),
+                    end_datetime_range_min=end_datetime_range_min.isoformat(),
+                    end_datetime_range_max=end_datetime_range_max.isoformat(),
+                    requires_update=False,
+                )
 
-                if (
-                    datetime_range
-                    and end_datetime_range
-                    and datetime_range.lower
-                    and datetime_range.upper
-                    and end_datetime_range.lower
-                    and end_datetime_range.upper
-                ):
-                    partition = Partition(
-                        name=partition_name,
-                        collection=item["collection"],
-                        datetime_range_min=datetime_range.lower.isoformat(),
-                        datetime_range_max=datetime_range.upper.isoformat(),
-                        end_datetime_range_min=end_datetime_range.lower.isoformat(),
-                        end_datetime_range_max=end_datetime_range.upper.isoformat(),
-                        requires_update=False,
-                    )
         else:
             partition = self._partition_cache[partition_name]
 
@@ -562,7 +575,7 @@ class Loader:
                     yield item
 
     def read_hydrated(
-        self, file: Union[Path, str, Iterator[Any]] = "stdin"
+        self, file: Union[Path, str, Iterator[Any]] = "stdin",
     ) -> Generator:
         for line in read_json(file):
             item = self.format_item(line)
@@ -604,8 +617,8 @@ class Loader:
         if not isinstance(_item, dict):
             try:
                 item = orjson.loads(str(_item).replace("\\\\", "\\"))
-            except:
-                raise Exception(f"Could not load {_item}")
+            except Exception:
+                raise
         else:
             item = _item
 
@@ -630,7 +643,7 @@ class Loader:
 
         if out["datetime"] is None or out["end_datetime"] is None:
             raise Exception(
-                f"Datetime must be set. OUT: {out} Properties: {properties}"
+                f"Datetime must be set. OUT: {out} Properties: {properties}",
             )
 
         if partition_trunc == "year":
